#define NEEDS_RNG
#include "stm32wrapper.h"

void clock_setup(void) {
    rcc_periph_clock_enable(RCC_GPIOD);
    rcc_periph_clock_enable(RCC_GPIOA);
    rcc_periph_clock_enable(RCC_USART2);
#ifdef NEEDS_RNG
    rcc_clock_setup_pll(&rcc_hse_8mhz_3v3[RCC_CLOCK_3V3_84MHZ]);
    rcc_periph_clock_enable(RCC_RNG);
#endif
}

void gpio_setup(void)
{
    /* Setup GPIO pin GPIO12 on GPIO port D for LED. */
    gpio_mode_setup(GPIOD, GPIO_MODE_OUTPUT, GPIO_PUPD_NONE, GPIO12);

    /* Setup GPIO pins for USART2 transmit. */
    gpio_mode_setup(GPIOA, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO2);

    /* Setup USART2 TX pin as alternate function. */
    gpio_set_af(GPIOA, GPIO_AF7, GPIO2);
}

void usart_setup(int baud)
{
    /* Setup USART2 parameters. */
    usart_set_baudrate(USART2, baud);
    usart_set_databits(USART2, 8);
    usart_set_stopbits(USART2, USART_STOPBITS_1);
    usart_set_mode(USART2, USART_MODE_TX);
    usart_set_parity(USART2, USART_PARITY_NONE);
    usart_set_flow_control(USART2, USART_FLOWCONTROL_NONE);

    /* Finally enable the USART. */
    usart_enable(USART2);
}

void flash_setup(void)
{
    FLASH_ACR |= (1 << 9);
    FLASH_ACR |= (1 << 10);
    FLASH_ACR |= FLASH_ACR_PRFTEN;
}

void send_USART_str(const char* in)
{
    int i;
    for(i = 0; in[i] != 0; i++) {
        gpio_toggle(GPIOD, GPIO12); /* LED on/off */
        usart_send_blocking(USART2, (unsigned char)in[i]);
        gpio_toggle(GPIOD, GPIO12); /* LED on/off */
    }
    usart_send_blocking(USART2, '\r');
    usart_send_blocking(USART2, '\n');
}

void send_USART_bytes(const unsigned char* in, int n)
{
    int i;
    for(i = 0; i < n; i++) {
        gpio_toggle(GPIOD, GPIO12); /* LED on/off */
        usart_send_blocking(USART2, in[i]);
        gpio_toggle(GPIOD, GPIO12); /* LED on/off */
    }
}

#ifdef NEEDS_RNG
inline void get_random(unsigned int* out, unsigned int len) {
    while (len > 0) {
        if ((RNG_SR & RNG_SR_DRDY) == 1)  // check if data is ready
            out[--len] = RNG_DR;
    }
}
#endif