#include "../common/stm32wrapper.h"
#include "aes.h"
#include <stdio.h>
#include <string.h>

int main(void) {
    clock_setup();
    gpio_setup();
    usart_setup(115200);
    flash_setup();

    SCS_DEMCR |= SCS_DEMCR_TRCENA;
    DWT_CTRL |= DWT_CTRL_CYCCNTENA;
    DWT_CYCCNT = 0;
    
    int oldcount, newcount;
    char buffer[100];
    uint32_t rkeys[120];
    unsigned char key[32];
    unsigned char ctext[32];

    // Values for test vector
    unsigned char ptext[32] = {
        0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
        0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff,
        0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
        0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff
    };

    for(int i = 0; i< 32; i++) {
        key[i] = i;
    }
    
    // AES-128 semi-fixsliced
    sprintf(buffer, "--- AES-128 semi-fixsliced ---");
    send_USART_str(buffer);

    // oldcount = DWT_CYCCNT;
    // aes128_keyschedule_sfs_lut(rkeys, key);
    // newcount = DWT_CYCCNT-oldcount;
    // sprintf(buffer, "cyc aes128_keyschedule_sfs_lut: %d", newcount);
    // send_USART_str(buffer);
    
    oldcount = DWT_CYCCNT;
    aes128_keyschedule_sfs(rkeys, key, key);
    newcount = DWT_CYCCNT-oldcount;
    sprintf(buffer, "cyc aes128_keyschedule_sfs: %d", newcount);
    send_USART_str(buffer);
    
    oldcount = DWT_CYCCNT;
    aes128_encrypt_sfs(ctext, ctext+16, ptext, ptext+16, rkeys);
    newcount = DWT_CYCCNT-oldcount;
    sprintf(buffer, "cyc aes128_encrypt_sfs: %d", newcount);
    send_USART_str(buffer);

    // AES-256 semi-fixsliced
    sprintf(buffer, "--- AES-256 semi-fixsliced ---");
    send_USART_str(buffer);

    // oldcount = DWT_CYCCNT;
    // aes256_keyschedule_sfs_lut(rkeys, key);
    // newcount = DWT_CYCCNT-oldcount;
    // sprintf(buffer, "cyc aes256_keyschedule_sfs_lut: %d", newcount);
    // send_USART_str(buffer);

    oldcount = DWT_CYCCNT;
    aes256_keyschedule_sfs(rkeys, key, key);
    newcount = DWT_CYCCNT-oldcount;
    sprintf(buffer, "cyc aes256_keyschedule_sfs: %d", newcount);
    send_USART_str(buffer);
    
    oldcount = DWT_CYCCNT;
    aes256_encrypt_sfs(ctext, ctext+16, ptext, ptext+16, rkeys);
    newcount = DWT_CYCCNT-oldcount;
    sprintf(buffer, "cyc aes256_encrypt_sfs: %d", newcount);
    send_USART_str(buffer);
    
    // AES-128 fully-fixsliced
    sprintf(buffer, "--- AES-128 fully-fixsliced ---");
    send_USART_str(buffer);

    // oldcount = DWT_CYCCNT;
    // aes128_keyschedule_ffs_lut(rkeys, key);
    // newcount = DWT_CYCCNT-oldcount;
    // sprintf(buffer, "cyc aes128_keyschedule_ffs_lut: %d", newcount);
    // send_USART_str(buffer);

    oldcount = DWT_CYCCNT;
    aes128_keyschedule_ffs(rkeys, key, key);
    newcount = DWT_CYCCNT-oldcount;
    sprintf(buffer, "cyc aes128_keyschedule_ffs: %d", newcount);
    send_USART_str(buffer);
    
    oldcount = DWT_CYCCNT;
    aes128_encrypt_ffs(ctext, ctext+16, ptext, ptext+16, rkeys);
    newcount = DWT_CYCCNT-oldcount;
    sprintf(buffer, "cyc aes128_encrypt_ffs: %d", newcount);
    send_USART_str(buffer);
    
    // AES-256 fully-fixsliced
    sprintf(buffer, "--- AES-256 fully-fixsliced ---");
    send_USART_str(buffer);

    // oldcount = DWT_CYCCNT;
    // aes256_keyschedule_ffs_lut(rkeys, key);
    // newcount = DWT_CYCCNT-oldcount;
    // sprintf(buffer, "cyc aes256_keyschedule_ffs_lut: %d", newcount);
    // send_USART_str(buffer);

    oldcount = DWT_CYCCNT;
    aes256_keyschedule_ffs(rkeys, key, key);
    newcount = DWT_CYCCNT-oldcount;
    sprintf(buffer, "cyc aes256_keyschedule_ffs: %d", newcount);
    send_USART_str(buffer);
    
    oldcount = DWT_CYCCNT;
    aes256_encrypt_ffs(ctext, ctext+16, ptext, ptext+16, rkeys);
    newcount = DWT_CYCCNT-oldcount;
    sprintf(buffer, "cyc aes256_encrypt_ffs: %d", newcount);
    send_USART_str(buffer);
    
    return 0;
}
